@@ -19,11 +19,12 @@ use constant _DUMP_TOKEN => scalar($DEBUG =~ /\b dump=token \b/xmsi);
 
 our @CARP_NOT = qw(Text::Xslate::Compiler Text::Xslate::Symbol);
 
-my $CODE    = qr/ (?: $STRING | [^'"] ) /xms;
+my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
+
 my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 # Operator tokens that the parser recognizes.
-# All the single characters are tokenized as an operator.
+# All the single characters are tokanized as an operator.
 my $OPERATOR_TOKEN = sprintf '(?:%s|[^ \t\r\n])', join('|', map{ quotemeta } qw(
     ...
     ..
@@ -207,7 +208,7 @@ sub parse {
 
     $parser->input( $parser->preprocess($input) );
 
-    $parser->next_token( $parser->tokenize() );
+    $parser->next_token( $parser->tokanize() );
     $parser->advance();
     my $ast = $parser->statements();
 
@@ -277,7 +278,7 @@ sub auto_chomp {
     return $nl;
 }
 
-# split templates by tags before tokenizing
+# split templates by tags before tokanizing
 sub split :method {
     my $parser  = shift;
     local($_) = @_;
@@ -290,6 +291,7 @@ sub split :method {
 
     my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
     my $lex_tag_start = qr/\A \Q$tag_start\E ($CHOMP_FLAGS?)/xms;
+    my $lex_tag_end   = qr/\A ($CODE) ($CHOMP_FLAGS?) \Q$tag_end\E/xms;
 
     my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= \Q$tag_start\E ) | \z ) ) /xms;
 
@@ -297,25 +299,10 @@ sub split :method {
 
     while($_) {
         if($in_tag) {
-            my $start = 0;
-            my $pos;
-            while( ($pos = index $_, $tag_end, $start) >= 0 ) {
-                my $code = substr $_, 0, $pos;
-                $code =~ s/$CODE//xmsog;
-                if(length($code) == 0) {
-                    last;
-                }
-                $start = $pos + 1;
-            }
-
-            if($pos >= 0) {
-                my $code = substr $_, 0, $pos, '';
-                $code =~ s/($CHOMP_FLAGS?) \z//xmso;
-                my $chomp = $1;
-
-                s/\A \Q$tag_end\E //xms or die "Oops!";
-
+            if(s/$lex_tag_end//xms) {
+                my($code, $chomp) = ($1, $2);
                 $in_tag = 0;
+
                 push @tokens, [ code => $code ];
                 if($chomp) {
                     push @tokens, [ postchomp => $chomp ];
@@ -566,9 +553,6 @@ sub init_symbols {
 
     $parser->symbol('include')  ->set_std(\&std_include);
 
-    $parser->symbol('last')  ->set_std(\&std_last);
-    $parser->symbol('next')  ->set_std(\&std_next);
-
     # macros
 
     $parser->symbol('cascade')  ->set_std(\&std_cascade);
@@ -577,7 +561,7 @@ sub init_symbols {
     $parser->symbol('before')   ->set_std(\&std_proc);
     $parser->symbol('after')    ->set_std(\&std_proc);
     $parser->symbol('block')    ->set_std(\&std_macro_block);
-    $parser->symbol('super')    ->set_std(\&std_super);
+    $parser->symbol('super')    ->set_std(\&std_marker);
     $parser->symbol('override') ->set_std(\&std_override);
 
     $parser->symbol('->')       ->set_nud(\&nud_lambda);
@@ -630,8 +614,8 @@ sub define_pair {
     return;
 }
 
-# the low-level tokenizer. Don't use it directly, use advance() instead.
-sub tokenize {
+# the low-level tokanizer. Don't use it directly, use advance() instead.
+sub tokanize {
     my($parser) = @_;
 
     local *_ = \$parser->{input};
@@ -668,7 +652,7 @@ sub next_token_is {
     return $parser->next_token->[1] eq $token;
 }
 
-# the high-level tokenizer
+# the high-level tokanizer
 sub advance {
     my($parser, $expect) = @_;
 
@@ -689,7 +673,7 @@ sub advance {
     $parser->statement_is_finished( $parser->following_newline != 0 );
     my $line = $parser->line( $parser->line + $parser->following_newline );
 
-    $parser->next_token( $parser->tokenize() );
+    $parser->next_token( $parser->tokanize() );
 
     my($arity, $id) = @{$t};
     if( $arity eq "name" && $parser->next_token_is("=>") ) {
@@ -1124,9 +1108,7 @@ sub define_function {
     my($parser, @names) = @_;
 
     foreach my $name(@names) {
-        my $s = $parser->symbol($name);
-        $s->set_nud(\&nud_name);
-        $s->is_defined(1);
+        $parser->symbol($name)->set_nud(\&nud_name);
     }
     return;
 }
@@ -1437,14 +1419,6 @@ sub std_for {
     my $proc = $symbol->clone(arity => 'for');
     $proc->first( $parser->expression(0) );
     $parser->pointy($proc, 1);
-    if($parser->token eq 'else') {
-        $parser->advance();
-        my $else = $parser->block();
-        $proc = $symbol->clone( arity => 'for_else',
-            first  => $proc,
-            second => $else,
-        )
-    }
     return $proc;
 }
 
@@ -1689,7 +1663,7 @@ sub barename {
     my($parser) = @_;
 
     my $t = $parser->token;
-    if($t->arity ne 'name' or $t->is_defined) {
+    if($t->arity ne 'name') {
         # string literal for 'cascade', or any expression for 'include'
         return $parser->expression(0);
     }
@@ -1763,21 +1737,10 @@ sub std_cascade {
     return $parser->finish_statement($stmt);
 }
 
-sub std_super {
+# markers for the compiler
+sub std_marker {
     my($parser, $symbol) = @_;
-    my $stmt = $symbol->clone(arity => 'super');
-    return $parser->finish_statement($stmt);
-}
-
-sub std_next {
-    my($parser, $symbol) = @_;
-    my $stmt = $symbol->clone(arity => 'loop_control', id => 'next');
-    return $parser->finish_statement($stmt);
-}
-
-sub std_last {
-    my($parser, $symbol) = @_;
-    my $stmt = $symbol->clone(arity => 'loop_control', id => 'last');
+    my $stmt = $symbol->clone(arity => 'marker');
     return $parser->finish_statement($stmt);
 }
 
@@ -1922,9 +1885,7 @@ sub make_alias { # alas(from => to)
 
     my $stash = $parser->symbol_table;
     if(exists $parser->symbol_table->{$to}) {
-        Carp::confess(
-            "Cannot make an alias to an existing symbol ($from => $to / "
-            . p($parser->symbol_table->{$to}) .")");
+        Carp::croak("Cannot make an alias to an existing symbol ($from => $to)");
     }
 
     # make a snapshot
